
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>
#include <stdarg.h>
#include <string.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <limits.h>
#include <dirent.h>
#include <ctype.h>
#include "error.h"


/* Kantenlaenge des Farbwuerfels */
#define MAXCOL 6


typedef struct{
        int x;
        int y;
       }store; 


/* Struktur zur Spriteverwaltung */

#define MAXSPRITEDATA 10

typedef struct XR_SPRITE {
        int priority;                            /* Prioritaet des Sprites */
                                                 /* hoechste Prioritaet ist 0 */
	int width;                               /* Bildbreite */
	int height;                              /* Bildhoehe  */
	int left, top;                           /* Position des Sprites */
	int xold, yold;
	int visible;                             /* sichtbar = 1, sonst 0 */
	int showed;                              /* schon gezeigt ? */
	struct XR_SPRITE *next;                  /* Naechstes Sprite */
	struct XR_SPRITE *prev;                  /* Vorheriges Sprite */
	unsigned char *SprPtr[MAXSPRITEDATA];    /* Zeiger auf Bilddaten */
	unsigned char *MskPtr[MAXSPRITEDATA];    /* Zeiger auf Bilddaten */
        int nSprDat;                             /* Anzahl der Spritebilder */
        int actSprDat;                           /* Aktuelles Spritebild */
        int vx, vy;                              /* Geschwindigkeit in x- und y-Richtung */
        int animcount;                           /* Interner Zaehler */
        int animtime;                            /* Wartezeit zum naechsten Bild */

	/* X Windows spezifisch */
	XImage *im;
	Colormap colormap;
}xr_Sprite;


/* Struktur zur Fensterverwaltung */

typedef struct{
       int           left, top;   /* Obere linke Fensterecke */
       int           width;       /* Fensterbreite */
       int           height;      /* Fensterhoehe */
       xr_Sprite     *first;      /* Erstes Sprite */
       
       /* X Windows-Spezifisch */ 
       Window        window;
       Pixmap        pixmap;
       Pixmap        background;
       Drawable      drawable;
       Colormap      colormap;
       unsigned long colref;
       store         start;
       store         end;
      }xr_Window;


/* Struktur zur Bildverwaltung */

typedef struct{
        int x, y;
	int width;                /* Bildbreite */
	int height;               /* Bildhoehe  */
	unsigned char *byteptr;   /* Zeiger auf Bilddaten */
	
	/* X Windows spezifisch */
	XImage *im;
	Colormap colormap;
}xr_Image;

/* Defines fuer Mausabfrage */
#define LEFTMOUSE     1
#define RIGHTMOUSE    2  
#define MIDDLEMOUSE   3

/* Defines fuer xr_Rectangle und xr_Ellipse */
#define NOTFILL 0
#define FILL   1

/* Defines fuer xr_Drawmode */
#define NORMAL 0
#define XOR    1
#define AND    2
#define OR     3

/* Defines fuer xr_Fade */
#define DORED   1 
#define DOGREEN 2
#define DOBLUE  4 

/* Extra Datentypen */
#define DWORD unsigned long
#define WORD  unsigned short

/* Fabtabelle fuer xr_PutImage */
#define RGB 1
#define DEF 0


/* Fensterfunktionen */

xr_Window *xr_CreateWindow(int left, int top, int width, int height); 
void xr_CloseWindow( );


/* Grundfunktionen erster Teil */

int  xr_Dither(int color);
void xr_SetPixel( int x, int y);
void xr_SetColor( int *rgb);
void xr_RGBMode( );
void xr_DrawMode( int mode);
int  xr_TextHeight( );
int  xr_TextWidth( );
void xr_Printf( const char *format, ...);


/* Grundfunktionen zweiter Teil */

int error(int errnum, char *funcname, char *info);
void xr_Move( int x, int y);
void xr_Line( int x, int y);
void xr_Rectangle( int left, int top, 
                  int right, int bottom, int fill);
void xr_Ellipse( int left, int top, 
                  int right, int bottom, int fill);


/* Grundfunktionen dritter Teil */

void xr_GetColor( int *rgb);
void xr_GetPixel( int x, int y, int *rgb);
void xr_Cls( );


/* Tastenfunktionen */

int  xr_GetKey( );
int  xr_WaitKey( );


/* Mausgrundfunktionen */

void xr_MousePos( int *x, int *y);
int xr_MouseButton( int which);


/* Fadeingfunktionen */

void xr_FadeOut( int steps);
void xr_Fade( unsigned char rgb_mask, float factor);


/* Bufferfunktionen */

void xr_DoubleBuffer( );
void xr_SingleBuffer( );
void xr_SwapBuffer( );


/* Dateifunktionen */

int xr_Read( char *filename, int x, int y);

int xr_ReadBMP( char *filename, int x, int y);
int xr_ReadGIF( char *filename, int x, int y);
int xr_ReadTIFF( char *filename, int x, int y);
int xr_ReadPCX( char *filename, int x, int y);
int xr_WriteBMP( char *filename, int x, int y, int width, int height);
int xr_WriteGIF( char *filename, int x, int y, int width, int height);
int xr_WriteTIFF( char *filename, int x, int y, int width, int height);
int xr_WritePCX( char *filename, int x, int y, int width, int height);


/* Imagefunktionen */

xr_Image *xr_CreateImage(int width, int height);
void xr_PutImage( xr_Image *im, int x, int y,
                 int width, int height, int steps);
xr_Image *xr_GetImage( int x, int y,
                 int width, int height);
void xr_DrawImage(xr_Image *dest, xr_Image *src, int x, int y,
                 int l1, int l2, int sb);
void xr_DestroyImage(xr_Image *im);                 


/* Imagedateifunktionen */

xr_Image *xr_ReadImage(char *Name);

xr_Image *xr_ReadBMPImage(char *filename);
xr_Image *xr_ReadGIFImage(char *filename);
xr_Image *xr_ReadTIFFImage(char *filename);
xr_Image *xr_ReadPCXImage(char *filename);
int xr_WriteBMPImage(xr_Image *im, char *filename);
int xr_WriteGIFImage(xr_Image *im, char *filename);
int xr_WriteTIFFImage(xr_Image *im, char *filename);
int xr_WritePCXImage(xr_Image *im, char *filename);


/* Animationsfunktionen */


int xr_GetFiles(char *pict_name, char *dir_name,
                 char *files[], int *nfiles);
int xr_MakeMovie(char *filename,
                 char *files[], int nfiles);
int xr_Animate( int show);
int xr_OpenMovie( char *filename);                
void xr_CloseMovie(void);
void xr_RemovePicture(void);
int xr_InsertPicture(char *filename);
int xr_SaveMovie(char *filename);


/* Imagegrafikfunktionen */

void xr_PutPixel(xr_Image *im, int x, int y, int rgb[]);                     


/* Spritefunktionen */

xr_Sprite *xr_GetSprite( char *files[], int nfiles, int pri);
void xr_MoveSprite( xr_Sprite *sp);
void xr_ShowSprites( );
void xr_ChangePriority( xr_Sprite *sp, int pri);
void xr_AnimateSprites( );
void xr_StartSpriteAnimation( );
void xr_EndSpriteAnimation( );
void xr_DestroySprite( xr_Sprite *sp);

/* Hilsfunktionen */
void get_n_bytes(unsigned char *bytepoint, unsigned char *bytes, int n);
DWORD getDWinWord(unsigned char *bytepoint, char x);
WORD getWinWord(unsigned char *bytepoint, char x);
DWORD getDWinWordFILE(FILE *pointer, char x);
WORD getWinWordFILE(FILE *pointer, char x);
void getBytes(DWORD word, unsigned char bytes[4]);
char *xr_GetString( int x, int y, int size);
